@@ -4,6 +4,8 @@
 #include <orbit/orb-core/orb-types.h>
 #include <orbit/orb-core/orbit-object.h>
 
+G_BEGIN_DECLS
+
 #if !defined(ORBIT_DECL_CORBA_ORB) && !defined(_CORBA_ORB_defined)
 #define ORBIT_DECL_CORBA_ORB 1
 #define _CORBA_ORB_defined 1
@@ -57,17 +59,22 @@ typedef struct CORBA_ORB_type *CORBA_ORB;
 
 struct CORBA_ORB_type {
 	struct ORBit_RootObject_struct  root_object;
+
+	GMutex                         *lock;
+
 	GIOPVersion                     default_giop_version;
 
 	GSList                         *servers;
+	GSList                         *profiles;
 	GPtrArray                      *adaptors;
 	GSList                         *current_invocations;
 	gpointer                        default_ctx;
 	GHashTable                     *initial_refs;
-	ORBit_genrand                   genrand;
 	guint                           life_flags;
 };
 
 #endif /* ORBIT2_INTERNAL_API */
 
+G_END_DECLS
+
 #endif
