@@ -1,3 +1,4 @@
+
 #include <config.h>
 #include <ctype.h>
 #include <string.h>
@@ -11,10 +12,12 @@
 #include "../util/orbit-purify.h"
 #include "iop-profiles.h"
 #include "orb-core-private.h"
+#if defined ENABLE_HTTP
 #include "orbhttp.h"
+#endif
 #include "orbit-debug.h"
 
-extern ORBit_option orbit_supported_options[];
+extern const ORBit_option orbit_supported_options[];
 
 #ifdef G_ENABLE_DEBUG
 OrbitDebugFlags _orbit_debug_flags = ORBIT_DEBUG_NONE;
@@ -29,6 +32,8 @@ static gboolean     orbit_use_usocks        = TRUE;
 static gboolean     orbit_use_irda          = FALSE;
 static gboolean     orbit_use_ssl           = FALSE;
 static gboolean     orbit_use_genuid_simple = FALSE;
+static gboolean     orbit_local_only       = FALSE;
+static gboolean     orbit_use_http_iors     = FALSE;
 static char        *orbit_ipsock            = NULL;
 static char        *orbit_ipname            = NULL;
 static char        *orbit_debug_options     = NULL;
@@ -36,7 +41,18 @@ static char        *orbit_debug_options     = NULL;
 void
 ORBit_ORB_start_servers (CORBA_ORB orb)
 {
-	LINCProtocolInfo *info;
+	LINCProtocolInfo     *info;
+	LINCConnectionOptions create_options = 0;
+
+	LINC_MUTEX_LOCK (orb->lock);
+	
+	if (orb->servers) { /* beaten to it */
+		LINC_MUTEX_UNLOCK (orb->lock);
+		return;
+	}
+
+	if (orbit_local_only)
+		create_options |= LINC_CONNECTION_LOCAL_ONLY;
 
 	for (info = linc_protocol_all (); info->name; info++) {
 		GIOPServer           *server;
@@ -44,9 +60,11 @@ ORBit_ORB_start_servers (CORBA_ORB orb)
 		if (!ORBit_proto_use (info->name))
 			continue;
 
-		server = giop_server_new (orb->default_giop_version,
-					  info->name, orbit_ipname,
-					  orbit_ipsock, 0, orb);
+		server = giop_server_new (
+			orb->default_giop_version, info->name,
+			orbit_ipname, orbit_ipsock,
+			create_options, orb);
+
 		if (server) {
 			orb->servers = g_slist_prepend (orb->servers, server);
 
@@ -56,7 +74,8 @@ ORBit_ORB_start_servers (CORBA_ORB orb)
 
 				server = giop_server_new (
 					orb->default_giop_version, info->name,
-					NULL, NULL, LINC_CONNECTION_SSL, orb);
+					NULL, NULL, LINC_CONNECTION_SSL | create_options,
+					orb);
 
 				if (server)
 					orb->servers = g_slist_prepend (orb->servers, server);
@@ -72,17 +91,25 @@ ORBit_ORB_start_servers (CORBA_ORB orb)
 	}
 
 	orb->profiles = IOP_start_profiles (orb);
+
+	LINC_MUTEX_UNLOCK (orb->lock);
 }
 
 static void
 ORBit_ORB_shutdown_servers (CORBA_ORB orb)
 {
+	LINC_MUTEX_LOCK (orb->lock);
+
 	IOP_shutdown_profiles (orb->profiles);
 	orb->profiles = NULL;
 
+	if (giop_threaded ())
+		g_warning ("FIXME: this needs delaying till worker thread exit");
 	g_slist_foreach (orb->servers, (GFunc) g_object_unref, NULL);
 	g_slist_free (orb->servers); 
 	orb->servers = NULL;
+
+	LINC_MUTEX_UNLOCK (orb->lock);
 }
 
 static ORBitGenUidType
@@ -100,6 +127,28 @@ ORBit_genuid_type (void)
 	return retval;
 }
 
+static void
+genuid_init (void)
+{
+	/* We treat the 'local_only' mode as a very special case */
+	if (orbit_local_only &&
+	    orbit_use_genuid_simple)
+		g_error  ("It is impossible to isolate one user from another "
+			  "with only simple cookie generation, you cannot "
+			  "explicitely enable this option and LocalOnly mode "
+			  "at the same time");
+
+	else if (!ORBit_genuid_init (ORBit_genuid_type ())) {
+
+		if (orbit_local_only)
+			g_error ("Failed to find a source of randomness good "
+				 "enough to insulate local users from each "
+				 "other. If you use Solaris you need /dev/random "
+				 "from the SUNWski package");
+	}
+}
+
+
 static void
 ORBit_service_list_free_ref (gpointer         key,
 			     ORBit_RootObject objref,
@@ -133,13 +182,15 @@ static void
 ORBit_setup_debug_flags (void)
 {
 	static GDebugKey debug_keys[] = {
-		{ "traces",   ORBIT_DEBUG_TRACES },
-		{ "timings",  ORBIT_DEBUG_TIMINGS },
-		{ "types",    ORBIT_DEBUG_TYPES },
-		{ "messages", ORBIT_DEBUG_MESSAGES },
-		{ "objects",  ORBIT_DEBUG_OBJECTS },
-		{ "giop",     ORBIT_DEBUG_GIOP },
-		{ "refs",     ORBIT_DEBUG_REFS },
+		{ "traces",        ORBIT_DEBUG_TRACES },
+		{ "inproc_traces", ORBIT_DEBUG_INPROC_TRACES },
+		{ "timings",       ORBIT_DEBUG_TIMINGS },
+		{ "types",         ORBIT_DEBUG_TYPES },
+		{ "messages",      ORBIT_DEBUG_MESSAGES },
+		{ "errors",        ORBIT_DEBUG_ERRORS },
+		{ "objects",       ORBIT_DEBUG_OBJECTS },
+		{ "giop",          ORBIT_DEBUG_GIOP },
+		{ "refs",          ORBIT_DEBUG_REFS },
 	};
 	const char *env_string;
 
@@ -156,6 +207,9 @@ ORBit_setup_debug_flags (void)
 			g_parse_debug_string (orbit_debug_options,
 					      debug_keys,
 					      G_N_ELEMENTS (debug_keys));
+
+	if (_orbit_debug_flags & ORBIT_DEBUG_INPROC_TRACES)
+		ORBit_small_flags &= ~ ORBIT_SMALL_FAST_LOCALS;
 }
 #endif /* G_ENABLE_DEBUG */
 
@@ -188,6 +242,7 @@ CORBA_ORB_init (int *argc, char **argv,
 		CORBA_ORBid orb_identifier,
 		CORBA_Environment *ev)
 {
+	gboolean threaded;
 	CORBA_ORB retval;
 	static ORBit_RootObject_Interface orb_if = {
 		ORBIT_ROT_ORB,
@@ -202,13 +257,20 @@ CORBA_ORB_init (int *argc, char **argv,
 	/* the allocation code uses the bottom bit of any pointer */
 	g_assert (ORBIT_ALIGNOF_CORBA_DOUBLE > 2);
 
+	threaded = (orb_identifier &&
+		    strstr (orb_identifier, "orbit-local-mt-orb") != NULL);
+
 	ORBit_option_parse (argc, argv, orbit_supported_options);
 
 #ifdef G_ENABLE_DEBUG
 	ORBit_setup_debug_flags ();
 #endif /* G_ENABLE_DEBUG */
 
-	giop_init ();
+	genuid_init ();
+	
+	giop_init (threaded,
+		   orbit_use_ipv4 || orbit_use_ipv6 ||
+		   orbit_use_irda || orbit_use_ssl);
 
 	ORBit_locks_initialize ();
 
@@ -217,9 +279,9 @@ CORBA_ORB_init (int *argc, char **argv,
 	ORBit_RootObject_init (&retval->root_object, &orb_if);
 	/* released by CORBA_ORB_destroy */
 	_ORBit_orb = ORBit_RootObject_duplicate (retval);
+	_ORBit_orb->lock = linc_mutex_new ();
 	g_atexit (shutdown_orb);
 
-	ORBit_genuid_init (ORBit_genuid_type ());
 	retval->default_giop_version = GIOP_LATEST;
 
 	retval->adaptors = g_ptr_array_new ();
@@ -230,8 +292,8 @@ CORBA_ORB_init (int *argc, char **argv,
 
 CORBA_char *
 CORBA_ORB_object_to_string (CORBA_ORB          orb,
-			   const CORBA_Object  obj,
-			   CORBA_Environment  *ev)
+			    const CORBA_Object obj,
+			    CORBA_Environment *ev)
 {
 	GIOPSendBuffer *buf;
 	CORBA_octet     endianness = GIOP_FLAG_ENDIANNESS;
@@ -301,13 +363,19 @@ CORBA_ORB_string_to_object (CORBA_ORB          orb,
 	int                  i;
 
 	if (strncmp (string, "IOR:", 4)) {
-		if (strstr (string, "://")) {
+
+#if defined ENABLE_HTTP
+		if (orbit_use_http_iors &&
+		    strstr (string, "://")) {
+			/* FIXME: this code is a security hazard */
 			ior = orb_http_resolve (string);
 			if (!ior)
 				return CORBA_OBJECT_NIL;
 
 			string = ior;
-		} else {
+		} else 
+#endif
+		{
 			CORBA_exception_set_system (
 					ev,
 					ex_CORBA_BAD_PARAM,
@@ -482,7 +550,7 @@ CORBA_ORB_resolve_initial_references (CORBA_ORB          orb,
 }
 
 static CORBA_TypeCode
-CORBA_TypeCode_allocate (void)
+ORBit_TypeCode_allocate (void)
 {
 	CORBA_TypeCode tc = g_new0 (struct CORBA_TypeCode_struct, 1);
 
@@ -501,17 +569,10 @@ CORBA_ORB_create_struct_tc (CORBA_ORB                    orb,
 	CORBA_TypeCode retval;
 	int            i;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval)
-		goto tc_alloc_failed;
+	retval = ORBit_TypeCode_allocate ();
 
 	retval->subtypes = g_new0 (CORBA_TypeCode, members->_length);
-	if (!retval->subtypes)
-		goto subtypes_alloc_failed;
-
 	retval->subnames = g_new0 (char *, members->_length);
-	if (!retval->subnames)
-		goto subnames_alloc_failed;
 
 	retval->kind      = CORBA_tk_struct;
 	retval->name      = g_strdup (name);
@@ -529,25 +590,18 @@ CORBA_ORB_create_struct_tc (CORBA_ORB                    orb,
 	}
 
 	return retval;
-
- subnames_alloc_failed:
-	g_free (retval->subtypes);
-
- subtypes_alloc_failed:
-	ORBit_RootObject_release (retval);
-
- tc_alloc_failed:
-	CORBA_exception_set_system (
-			ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-	return CORBA_OBJECT_NIL;
 }
 
 static void
 copy_case_value (CORBA_long *dest,
 		 CORBA_any  *src)
 {
-        switch (src->_type->kind) {
+	CORBA_TypeCode tc = src->_type;
+
+	if (tc->kind == CORBA_tk_alias)
+		tc = tc->subtypes [0];
+
+        switch (tc->kind) {
         case CORBA_tk_ulong:
         case CORBA_tk_long:
         case CORBA_tk_enum:
@@ -579,24 +633,13 @@ CORBA_ORB_create_union_tc (CORBA_ORB                   orb,
 	CORBA_TypeCode retval;
 	int            i;
 
-	retval = CORBA_TypeCode_allocate ();
-
-	if (!retval)
-		goto tc_alloc_failed;
+	retval = ORBit_TypeCode_allocate ();
 
 	retval->discriminator = ORBit_RootObject_duplicate (discriminator_type);
 		
-	retval->subtypes = g_new0 (CORBA_TypeCode, members->_length);
-	if (!retval)
-		goto subtypes_alloc_failed;
-
-	retval->subnames = g_new0 (char *, members->_length);
-	if(!retval->subnames)
-		goto subnames_alloc_failed;
-
+	retval->subtypes  = g_new0 (CORBA_TypeCode, members->_length);
+	retval->subnames  = g_new0 (char *, members->_length);
 	retval->sublabels = g_new0 (CORBA_long, members->_length);
-	if (!retval->sublabels)
-		goto sublabels_alloc_failed;
 
 	retval->kind          = CORBA_tk_union;
 	retval->name          = g_strdup (name);
@@ -620,22 +663,6 @@ CORBA_ORB_create_union_tc (CORBA_ORB                   orb,
 	}
 
 	return retval;
-
- sublabels_alloc_failed:
-	g_free (retval->sublabels);
-
- subnames_alloc_failed:
-	g_free (retval->subtypes);
-
- subtypes_alloc_failed:
-	ORBit_free (retval->discriminator);
-	ORBit_RootObject_release (retval);
-
- tc_alloc_failed:
-	CORBA_exception_set_system (
-			ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-	return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
@@ -648,13 +675,9 @@ CORBA_ORB_create_enum_tc (CORBA_ORB                  orb,
 	CORBA_TypeCode retval;
 	int            i;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval)
-		goto tc_alloc_failed;
+	retval = ORBit_TypeCode_allocate ();
 
 	retval->subnames=g_new0 (char *, members->_length);
-	if (!retval->subnames)
-		goto subnames_alloc_failed;
 
 	retval->kind      = CORBA_tk_enum;
 	retval->name      = g_strdup (name);
@@ -666,15 +689,6 @@ CORBA_ORB_create_enum_tc (CORBA_ORB                  orb,
 		retval->subnames [i] = g_strdup (members->_buffer [i]);
 
 	return retval;
-
- subnames_alloc_failed:
-	ORBit_RootObject_release (retval);
-
- tc_alloc_failed:
-	CORBA_exception_set_system (
-			ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-	return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
@@ -686,13 +700,9 @@ CORBA_ORB_create_alias_tc (CORBA_ORB             orb,
 {
 	CORBA_TypeCode retval;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval)
-		goto tc_alloc_failed;
+	retval = ORBit_TypeCode_allocate ();
 	
 	retval->subtypes = g_new0 (CORBA_TypeCode, 1);
-	if (!retval->subtypes)
-		goto subtypes_alloc_failed;
 
 	retval->kind      = CORBA_tk_alias;
 	retval->name      = g_strdup (name);
@@ -703,15 +713,6 @@ CORBA_ORB_create_alias_tc (CORBA_ORB             orb,
 	retval->subtypes [0] = ORBit_RootObject_duplicate (original_type);
 
 	return retval;
-
- subtypes_alloc_failed:
-	ORBit_RootObject_release (retval);
-
- tc_alloc_failed:
-	CORBA_exception_set_system (
-			ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-	return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
@@ -724,18 +725,11 @@ CORBA_ORB_create_exception_tc (CORBA_ORB                    orb,
 	CORBA_TypeCode retval;
 	int            i;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval)
-		goto tc_alloc_failed;
+	retval = ORBit_TypeCode_allocate ();
 
 	if (members->_length) {
 		retval->subtypes = g_new0 (CORBA_TypeCode, members->_length);
-		if (!retval->subtypes)
-			goto subtypes_alloc_failed;
-
 		retval->subnames = g_new0 (char *, members->_length);
-		if (!retval->subnames)
-			goto subnames_alloc_failed;
 	}
 
 	retval->kind      = CORBA_tk_except;
@@ -754,18 +748,6 @@ CORBA_ORB_create_exception_tc (CORBA_ORB                    orb,
 	}
 
 	return retval;
-
- subnames_alloc_failed:
-	g_free (retval->subtypes);
-
- subtypes_alloc_failed:
-	ORBit_RootObject_release (retval);
-
- tc_alloc_failed:
-	CORBA_exception_set_system (
-			ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-	return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
@@ -776,13 +758,7 @@ CORBA_ORB_create_interface_tc (CORBA_ORB                 orb,
 {
 	CORBA_TypeCode retval;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval) {
-		CORBA_exception_set_system (
-				ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-		return CORBA_OBJECT_NIL;
-	}
+	retval = ORBit_TypeCode_allocate ();
 
 	retval->kind    = CORBA_tk_objref;
 	retval->name    = g_strdup (name);
@@ -798,13 +774,7 @@ CORBA_ORB_create_string_tc (CORBA_ORB                  orb,
 {
 	CORBA_TypeCode retval;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval) {
-		CORBA_exception_set_system (
-				ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-		return CORBA_OBJECT_NIL;
-	}
+	retval = ORBit_TypeCode_allocate ();
 
 	retval->kind   = CORBA_tk_string;
 	retval->length = bound;
@@ -819,13 +789,7 @@ CORBA_ORB_create_wstring_tc (CORBA_ORB                  orb,
 {
 	CORBA_TypeCode retval;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval) {
-		CORBA_exception_set_system (
-				ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-		return CORBA_OBJECT_NIL;
-	}
+	retval = ORBit_TypeCode_allocate ();
 
 	retval->kind   = CORBA_tk_wstring;
 	retval->length = bound;
@@ -841,13 +805,7 @@ CORBA_ORB_create_fixed_tc (CORBA_ORB                   orb,
 {
 	CORBA_TypeCode retval;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval) {
-		CORBA_exception_set_system (
-				ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-		return CORBA_OBJECT_NIL;
-	}
+	retval = ORBit_TypeCode_allocate ();
 
 	retval->kind   = CORBA_tk_fixed;
 	retval->digits = digits;
@@ -864,13 +822,9 @@ CORBA_ORB_create_sequence_tc (CORBA_ORB                  orb,
 {
 	CORBA_TypeCode retval;
 
-	retval = CORBA_TypeCode_allocate ();
-	if (!retval)
-		goto tc_alloc_failed;
+	retval = ORBit_TypeCode_allocate ();
 
 	retval->subtypes = g_new0 (CORBA_TypeCode, 1);
-	if (!retval->subtypes)
-		goto subtypes_alloc_failed;
 
 	retval->kind      = CORBA_tk_sequence;
 	retval->sub_parts = 1;
@@ -879,15 +833,6 @@ CORBA_ORB_create_sequence_tc (CORBA_ORB                  orb,
 	retval->subtypes [0] = ORBit_RootObject_duplicate (element_type);
 
 	return retval;
-
- subtypes_alloc_failed:
-	ORBit_RootObject_release (retval);
-
- tc_alloc_failed:
-	CORBA_exception_set_system (
-			ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-	return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
@@ -898,33 +843,20 @@ CORBA_ORB_create_recursive_sequence_tc (CORBA_ORB                  orb,
 {
 	CORBA_TypeCode retval;
 
-	retval=CORBA_TypeCode_allocate ();
-	if (retval)
-		goto tc_alloc_failed;
+	retval=ORBit_TypeCode_allocate ();
 
 	retval->subtypes = g_new0 (CORBA_TypeCode, 1);
-	if (!retval->subtypes)
-		goto subtypes_alloc_failed;
 
 	retval->kind      = CORBA_tk_sequence;
 	retval->sub_parts = 1;
 	retval->length    = bound;
 
-	retval->subtypes [0] = CORBA_TypeCode_allocate ();
+	retval->subtypes [0] = ORBit_TypeCode_allocate ();
 
 	retval->subtypes [0]->kind          = CORBA_tk_recursive;
 	retval->subtypes [0]->recurse_depth = offset;
 
 	return retval;
-
- subtypes_alloc_failed:
-	ORBit_RootObject_release (retval);
-
- tc_alloc_failed:
-	CORBA_exception_set_system (
-			ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-	return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
@@ -935,13 +867,9 @@ CORBA_ORB_create_array_tc (CORBA_ORB                  orb,
 {
 	CORBA_TypeCode tc;
 
-	tc = CORBA_TypeCode_allocate ();
-	if (!tc)
-		goto tc_alloc_failed;
+	tc = ORBit_TypeCode_allocate ();
 
 	tc->subtypes = g_new0 (CORBA_TypeCode, 1);
-	if (!tc->subtypes)
-		goto subtypes_alloc_failed;
 
 	tc->kind      = CORBA_tk_array;
 	tc->sub_parts = 1;
@@ -950,15 +878,6 @@ CORBA_ORB_create_array_tc (CORBA_ORB                  orb,
 	tc->subtypes [0] = ORBit_RootObject_duplicate (element_type);
 
 	return (tc);
-
- subtypes_alloc_failed:
-	ORBit_RootObject_release (tc);
-
- tc_alloc_failed:
-	CORBA_exception_set_system (
-			ev, ex_CORBA_NO_MEMORY, CORBA_COMPLETED_NO);
-
-	return CORBA_OBJECT_NIL;
 }
 
 CORBA_TypeCode
@@ -1050,9 +969,7 @@ CORBA_ORB_shutdown (CORBA_ORB           orb,
 
 	ORBit_ORB_shutdown_servers (orb);
 
-	giop_connection_remove_by_orb (orb);
-
-	g_main_loop_quit (linc_loop);
+	giop_shutdown ();
 }
 
 void
@@ -1079,9 +996,11 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 	root_poa = g_ptr_array_index (orb->adaptors, 0);
 	if (root_poa &&
 	    ((ORBit_RootObject) root_poa)->refs != 1) {
+#ifdef G_ENABLE_DEBUG
 		g_warning ("CORBA_ORB_destroy: Application still has %d "
 			   "refs to RootPOA.",
 			   ((ORBit_RootObject)root_poa)->refs - 1);
+#endif
 		CORBA_exception_set_system (
 			ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
 	}
@@ -1108,14 +1027,21 @@ CORBA_ORB_destroy (CORBA_ORB          orb,
 		}
 
 		if (leaked_adaptors) {
+#ifdef G_ENABLE_DEBUG
 			g_warning ("CORBA_ORB_destroy: leaked '%d' Object Adaptors", leaked_adaptors);
+#endif
 			CORBA_exception_set_system (
 				ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
 		}
 
 		if (((ORBit_RootObject)orb)->refs != 2 + leaked_adaptors) {
-			g_warning ("CORBA_ORB_destroy: ORB still has %d refs.",
-				   ((ORBit_RootObject)orb)->refs - 1 - leaked_adaptors);
+#ifdef G_ENABLE_DEBUG
+			if (((ORBit_RootObject)orb)->refs == 1 + leaked_adaptors)
+				g_warning ("CORBA_ORB_destroy: ORB unreffed but not _destroy'd");
+			else
+				g_warning ("CORBA_ORB_destroy: ORB still has %d refs.",
+					   ((ORBit_RootObject)orb)->refs - 1 - leaked_adaptors);
+#endif
 			CORBA_exception_set_system (
 				ev, ex_CORBA_FREE_MEM, CORBA_COMPLETED_NO);
 		}
@@ -1204,21 +1130,26 @@ ORBit_proto_use (const char *name)
 	return FALSE;
 }
 
-static ORBit_option orbit_supported_options[] = {
-	{"ORBid",           ORBIT_OPTION_STRING,  NULL}, /* FIXME: unimplemented */
-	{"ORBImplRepoIOR",  ORBIT_OPTION_STRING,  NULL}, /* FIXME: unimplemented */
-	{"ORBIfaceRepoIOR", ORBIT_OPTION_STRING,  NULL}, /* FIXME: unimplemented */
-	{"ORBNamingIOR",    ORBIT_OPTION_STRING,  NULL}, /* FIXME: unimplemented */
-	{"ORBRootPOAIOR",   ORBIT_OPTION_STRING,  NULL}, /* FIXME: huh?          */
- 	{"ORBIIOPIPName",   ORBIT_OPTION_STRING,  &orbit_ipname},
- 	{"ORBIIOPIPSock",   ORBIT_OPTION_STRING,  &orbit_ipsock},
-	{"ORBIIOPIPv4",     ORBIT_OPTION_BOOLEAN, &orbit_use_ipv4},
-	{"ORBIIOPIPv6",     ORBIT_OPTION_BOOLEAN, &orbit_use_ipv6},
-	{"ORBIIOPUSock",    ORBIT_OPTION_BOOLEAN, &orbit_use_usocks},
-	{"ORBIIOPUNIX",     ORBIT_OPTION_BOOLEAN, &orbit_use_usocks},
-	{"ORBIIOPIrDA",     ORBIT_OPTION_BOOLEAN, &orbit_use_irda},
-	{"ORBIIOPSSL",      ORBIT_OPTION_BOOLEAN, &orbit_use_ssl},
-	{"ORBSimpleUIDs",   ORBIT_OPTION_BOOLEAN, &orbit_use_genuid_simple},
-	{"ORBDebugFlags",   ORBIT_OPTION_STRING,  &orbit_debug_options},
-	{NULL,              0,                    NULL},
+const ORBit_option orbit_supported_options[] = {
+	{ "ORBid",           ORBIT_OPTION_STRING,  NULL }, /* FIXME: unimplemented */
+	{ "ORBImplRepoIOR",  ORBIT_OPTION_STRING,  NULL }, /* FIXME: unimplemented */
+	{ "ORBIfaceRepoIOR", ORBIT_OPTION_STRING,  NULL }, /* FIXME: unimplemented */
+	{ "ORBNamingIOR",    ORBIT_OPTION_STRING,  NULL }, /* FIXME: unimplemented */
+	{ "ORBRootPOAIOR",   ORBIT_OPTION_STRING,  NULL }, /* FIXME: huh?          */
+ 	{ "ORBIIOPIPName",   ORBIT_OPTION_STRING,  &orbit_ipname },
+ 	{ "ORBIIOPIPSock",   ORBIT_OPTION_STRING,  &orbit_ipsock },
+	{ "ORBLocalOnly",    ORBIT_OPTION_BOOLEAN, &orbit_local_only },
+	/* warning: this option is a security risk unless used with LocalOnly */
+	{ "ORBIIOPIPv4",     ORBIT_OPTION_BOOLEAN, &orbit_use_ipv4 },
+	{ "ORBIIOPIPv6",     ORBIT_OPTION_BOOLEAN, &orbit_use_ipv6 },
+	{ "ORBIIOPUSock",    ORBIT_OPTION_BOOLEAN, &orbit_use_usocks },
+	{ "ORBIIOPUNIX",     ORBIT_OPTION_BOOLEAN, &orbit_use_usocks },
+	{ "ORBIIOPIrDA",     ORBIT_OPTION_BOOLEAN, &orbit_use_irda },
+	{ "ORBIIOPSSL",      ORBIT_OPTION_BOOLEAN, &orbit_use_ssl },
+	/* warning: this option is a security risk */
+	{ "ORBHTTPIORs",     ORBIT_OPTION_BOOLEAN, &orbit_use_http_iors },
+	/* warning: this option is a security risk */
+	{ "ORBSimpleUIDs",   ORBIT_OPTION_BOOLEAN, &orbit_use_genuid_simple },
+	{ "ORBDebugFlags",   ORBIT_OPTION_STRING,  &orbit_debug_options },
+	{ NULL,              0,                    NULL },
 };
