@@ -163,10 +163,11 @@ ORBit_handle_exception_array (GIOPRecvBuffer     *rb,
 				ev->_any._type = ORBit_RootObject_duplicate (
 					types->_buffer [i]);
 		}
-		return;
 	}
-	return;
   
+	if (ev->_major != CORBA_NO_EXCEPTION)
+		return;
+
  errout:
 	/* ignore LOCATION_FORWARD here, that gets handled in the stub */
 	CORBA_exception_set_system (ev, ex_CORBA_MARSHAL,
@@ -219,7 +220,7 @@ ORBit_small_marshal_context (GIOPSendBuffer *send_buffer,
 	   efficiency of the current impl */
 	ORBit_ContextMarshalItem *mlist;
 
-	mlist = alloca (sizeof (ORBit_ContextMarshalItem) *
+	mlist = g_alloca (sizeof (ORBit_ContextMarshalItem) *
 			m_data->contexts._length);
 
 	tprintf (" context { ");
@@ -283,7 +284,8 @@ typedef struct {
 #define do_marshal_value(a,b,c)     \
 	ORBit_marshal_value   ((a),(gconstpointer *)(b),(c))
 #define do_demarshal_value(a,b,c,e) \
-	ORBit_demarshal_value ((c),(b),(a),(e))
+	if (ORBit_demarshal_value ((c),(b),(a),(e))) \
+		goto demarshal_exception
 
 static gboolean
 orbit_small_marshal (CORBA_Object           obj,
@@ -304,7 +306,7 @@ orbit_small_marshal (CORBA_Object           obj,
 	{
 		int          align;
 		int          len = sizeof (CORBA_unsigned_long) + m_data->name_len + 1;
-		guchar      *header = alloca (len + sizeof (CORBA_unsigned_long));
+		guchar      *header = g_alloca (len + sizeof (CORBA_unsigned_long));
 
 		*(CORBA_unsigned_long *) header = m_data->name_len + 1;
 		memcpy (header + sizeof (CORBA_unsigned_long),
@@ -328,8 +330,7 @@ orbit_small_marshal (CORBA_Object           obj,
 
 	dprintf (MESSAGES, "Marshal: id 0x%x\n", request_id);
 
-	for (i = 0; (i < m_data->arguments._length &&
-		     m_data->arguments._buffer [i].flags); i++) {
+	for (i = 0; i < m_data->arguments._length; i++) {
 
 		ORBit_IArg *a = &m_data->arguments._buffer [i];
 		gpointer    p;
@@ -350,7 +351,7 @@ orbit_small_marshal (CORBA_Object           obj,
 		p = args [i];
 		do_marshal_value (send_buffer, &p, tc);
 
-		if (m_data->arguments._buffer [i+1].flags)
+		if (i < m_data->arguments._length - 1)
 			tprintf (", ");
 	}
 
@@ -400,7 +401,7 @@ orbit_small_demarshal (CORBA_Object           obj,
 	GIOPRecvBuffer *recv_buffer;
 	CORBA_ORB       orb = obj->orb;
 
-	recv_buffer = giop_recv_buffer_get (mqe, TRUE);
+	recv_buffer = giop_recv_buffer_get (mqe);
 	if (!recv_buffer) {
 		dprintf (MESSAGES, "No recv buffer ...\n");
 		return MARSHAL_SYS_EXCEPTION_INCOMPLETE;
@@ -409,7 +410,7 @@ orbit_small_demarshal (CORBA_Object           obj,
 	if (giop_recv_buffer_reply_status (recv_buffer) != GIOP_NO_EXCEPTION)
  		goto msg_exception;
 
-	if ((tc = m_data->ret)) {
+	if ((tc = m_data->ret) && tc->kind != CORBA_tk_void) {
 		tprintf (" =>: ");
 
 		g_assert (ret != NULL);
@@ -452,8 +453,7 @@ orbit_small_demarshal (CORBA_Object           obj,
 		int i;
 		int trace_have_out = 0;
 
-		for (i = 0; (i < m_data->arguments._length &&
-			     m_data->arguments._buffer [i].flags); i++) {
+		for (i = 0; i < m_data->arguments._length; i++) {
 			const ORBit_IArg *a;
 			gpointer          arg;
 
@@ -515,7 +515,7 @@ orbit_small_demarshal (CORBA_Object           obj,
 				break;
 			}
 			if (trace_have_out &&
-			    m_data->arguments._buffer [i + 1].flags)
+			    i < m_data->arguments._length - 1)
 				tprintf (", ");
 		}
 		if (trace_have_out)
@@ -527,6 +527,12 @@ orbit_small_demarshal (CORBA_Object           obj,
 
 	return MARSHAL_CLEAN;
 
+ demarshal_exception:
+	/* FIXME: may well leak */
+	CORBA_exception_set_system(ev, ex_CORBA_MARSHAL,
+				   CORBA_COMPLETED_MAYBE);
+	return MARSHAL_EXCEPTION_COMPLETE;
+
  msg_exception:
 	if (giop_recv_buffer_reply_status (recv_buffer) ==
 	    GIOP_LOCATION_FORWARD) {
@@ -691,7 +697,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 
 	tprintf_header (adaptor_obj->objref, m_data);
 
-	if ((tc = m_data->ret)) {
+	if ((tc = m_data->ret) && tc->kind != CORBA_tk_void) {
 		
 		while (tc->kind == CORBA_tk_alias)
 			tc = tc->subtypes [0];
@@ -699,7 +705,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 		switch (tc->kind) {
 		case BASE_TYPES:
 		case OBJ_STRING_TYPES:
-			retval = alloca (ORBit_gather_alloc_info (tc));
+			retval = g_alloca (ORBit_gather_alloc_info (tc));
 			break;
 		case STRUCT_UNION_TYPES:
 			if (m_data->flags & ORBit_I_COMMON_FIXED_SIZE) {
@@ -717,8 +723,10 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 		int len = m_data->arguments._length *
 			sizeof (gpointer);
 
-		args = alloca (len);
-		scratch = alloca (len);
+		args = g_alloca (len);
+		memset (args, 0, len);
+		scratch = g_alloca (len);
+		memset (scratch, 0, len);
 	}		
 
 	for (i = 0; i < m_data->arguments._length; i++) {
@@ -736,7 +744,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			switch (tc->kind) {
 			case BASE_TYPES:
 			case OBJ_STRING_TYPES:
-				p = args [i] = alloca (ORBit_gather_alloc_info (tc));
+				p = args [i] = g_alloca (ORBit_gather_alloc_info (tc));
 				do_demarshal_value (recv_buffer, &p, tc, orb);
 
 				p = args [i];
@@ -745,7 +753,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			case STRUCT_UNION_TYPES:
 			case CORBA_tk_array:
 				if (a->flags & ORBit_I_COMMON_FIXED_SIZE) {
-					p = args [i] = alloca (ORBit_gather_alloc_info (tc));
+					p = args [i] = g_alloca (ORBit_gather_alloc_info (tc));
 					do_demarshal_value (recv_buffer, &p, tc, orb);
 					p = args [i];
 					tprintf_trace_value (&p, tc);
@@ -769,7 +777,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			switch (tc->kind) {
 			case BASE_TYPES:
 			case OBJ_STRING_TYPES:
-				scratch [i] = alloca (ORBit_gather_alloc_info (tc));
+				scratch [i] = g_alloca (ORBit_gather_alloc_info (tc));
 				break;
 			case STRUCT_UNION_TYPES:
 			case CORBA_tk_array:
@@ -784,7 +792,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 
 		}
 
-		if (m_data->arguments._buffer [i+1].flags)
+		if (i < m_data->arguments._length - 1)
 			tprintf (", ");
 	}
 
@@ -803,8 +811,14 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 
 	if (m_data->flags & ORBit_I_METHOD_1_WAY)
 		goto clean_out;
+	else
+		goto handle_possible_exception;
+	
+ demarshal_exception:
+	CORBA_exception_set_system (ev, ex_CORBA_MARSHAL,
+				    CORBA_COMPLETED_NO);
 
- sys_exception:
+ handle_possible_exception:
 	/* FIXME: should we be using the connection's GIOP version ? */
 	send_buffer = giop_send_buffer_use_reply (
 		recv_buffer->giop_version,
@@ -822,7 +836,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			   so we throw a system exception next */
 			dprintf (MESSAGES, "Re-sending an exception, this time %d: '%s'",
 				 ev->_major, ev->_id);
-			goto sys_exception;
+			goto handle_possible_exception;
 		}
 		tprintf ("User exception '%s'", ev->_id);
 
@@ -834,7 +848,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 	} else { /* Marshal return values */
 		int trace_have_out = 0;
 
-		if ((tc = m_data->ret)) {
+		if ((tc = m_data->ret) && tc->kind != CORBA_tk_void) {
 			gpointer p = retval;
 
 			tprintf (" =>; ");
@@ -896,6 +910,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			}
 
 			if (trace_have_out &&
+			    i < m_data->arguments._length - 1 &&
 			    m_data->arguments._buffer [i + 1].flags &
 			    (ORBit_I_ARG_OUT | ORBit_I_ARG_INOUT))
 				tprintf (", ");
@@ -909,7 +924,8 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 	giop_send_buffer_write (send_buffer, recv_buffer->connection, FALSE);
 	giop_send_buffer_unuse (send_buffer);
 
-	if (m_data->ret && ev->_major == CORBA_NO_EXCEPTION) {
+	if (m_data->ret && tc->kind != CORBA_tk_void &&
+	    ev->_major == CORBA_NO_EXCEPTION) {
 		switch (m_data->ret->kind) {
 		case BASE_TYPES:
 			break;
@@ -1075,7 +1091,7 @@ async_recv_cb (ORBitAsyncQueueEntry *aqe)
 	CORBA_exception_init (ev);
 
 	/* So we don't get invoked again */
-	aqe->mqe.u.unthreaded.cb = NULL;
+	aqe->mqe.async_cb = NULL;
 
 	if (!aqe->mqe.cnx ||
 	    aqe->mqe.cnx->parent.status == LINC_DISCONNECTED)
