@@ -1,26 +1,21 @@
+#ifdef G_ENABLE_DEBUG
+
 #include "config.h"
-#include <orbit/orbit.h>
+#include <unistd.h>
 #include <string.h>
+#include <sys/time.h>
+#include <ctype.h>
+#include <orbit/orbit.h>
 
 #include "orb-core-private.h"
 #include "orbit-debug.h"
 
-#ifdef TRACE_DEBUG
-#  include <unistd.h>
-#endif
-
-#ifdef TRACE_TIMING
-#  include <sys/time.h>
-#endif
-
 /*
  * Flip this switch to debug the
  * debug code.
  */
 #undef DEBUG_TRACE
 
-
-#ifdef TRACE_DEBUG
 #define NOT_NULL(str) ((str) == NULL ? "(null)" : (str))
 
 void
@@ -149,15 +144,31 @@ ORBit_trace_value (gconstpointer *val, CORBA_TypeCode tc)
 		break;
 
 	case CORBA_tk_string: {
-		char *p, *str = g_strdup (*(char **)*val);
+		if (val == NULL)
+			tprintf("(null)");
+		else {
+			const int max = 64;
+			const char * v = (*(const char **)*val);
+			char *p, *str = g_strndup (v, max);
+			int len = strlen (v);
 
-		for (p = str; p && *p; p++) {
-			if (!isascii ((int)*p))
-				*p = '#';
+			if (len > max) {
+				const char ins[5] = " ... ";
+				int i;
+
+				for (i = 0; i < max / 2; ++i)
+					str[i + max / 2] = v[len - max / 2 + i];
+				strncpy(str + (max - sizeof(ins)) / 2, ins, sizeof(ins));
+			}
+
+			for (p = str; p && *p; p++)
+				if (!isascii ((int)*p))
+					*p = '#';
+			
+			tprintf ("'%s'", str);
+			
+			g_free (str);
 		}
-		tprintf ("'%s'", NOT_NULL (str));
-
-		g_free (str);
 		break;
 	}
 
@@ -211,16 +222,14 @@ ORBit_trace_value (gconstpointer *val, CORBA_TypeCode tc)
 	*val = ((guchar *)*val) + ORBit_gather_alloc_info (tc);
 }
 
-#ifdef TRACE_TIMING
 void
 ORBit_trace_timestamp (void)
 {
 	struct timeval t;
 
 	gettimeofday (&t, NULL);
-	tprintf ("%lu.%lu ", t.tv_sec, t.tv_usec);
+	tprintf ("%lu.%06lu ", t.tv_sec, t.tv_usec);
 }
-#endif
 
 void
 ORBit_trace_header (CORBA_Object   object,
@@ -243,4 +252,23 @@ ORBit_trace_end_method (void)
 	tprintf ("\n");
 }
 
-#endif /* TRACE_DEBUG */
+void
+ORBit_trace_profiles (CORBA_Object obj)
+{
+	tprintf ("p %d: Obj %p (%s) profiles: ",
+		 getpid (), obj,
+		 g_quark_to_string (obj ? obj->type_qid : 0));
+	if (obj) {
+		GSList *l;
+
+		for (l = obj->profile_list; l; l = l->next) {
+			char *s;
+			s = IOP_profile_dump (obj, l->data);
+			tprintf ("'%s' ", s);
+			g_free (s);
+		}
+	}
+	tprintf ("\n");
+}
+
+#endif /* G_ENABLE_DEBUG */
